@@ -65,24 +65,21 @@ class Events:
             ValueError: Days for retention must be greater than or equal to 0
         """
 
-        if event_archive and event_archive < 0:
-            # If intending to create an event archive check to make sure days is not less than zero
+        if event_archive and archive_days < 0:
             raise ValueError("Days for Archive Retention must be greater than or equal to 0 (zero).")
-        if event_archive and event_archive == 0:
-            # if intending to create an archive, warn if days is equal to zero
+        if event_archive and archive_days == 0:
             print("You enterd 0 (zero) for the amount of days to retain messages in the archive. This will result in events being stored indefinitely so use caution.")
 
         # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventbus/
         bus = aws.cloudwatch.EventBus(f"{name}-bus")
-
         pulumi.export(f"{name}-bus-arn", bus.arn)
 
         if event_archive:
             if isinstance(archive_event_pattern, dict):
-                # Checking to see if the archive pattern was sent as a dict, if so dump to string
                 print("The archive event pattern is a dictionary and will be converted to a string.")
                 archive_event_pattern = json.dumps(archive_event_pattern)
 
+            # https://www.pulumi.com/registry/packages/aws-native/api-docs/events/archive/
             event_archive = aws_native.events.Archive(f"{name}-event-archive",
                 description=f"Archived events from {name}-bus",
                 event_source_arn=bus.arn,
@@ -90,7 +87,6 @@ class Events:
                 event_pattern=archive_event_pattern,
                 opts = pulumi.ResourceOptions(parent=bus)
             )
-
             pulumi.export(f"{name}-event-archive-arn", event_archive.arn)
 
         if schema_discoverer:
@@ -99,9 +95,10 @@ class Events:
                 source_arn=bus.arn,
                 description=f"Schema Discover for the {name}-bus",
             )
-
             pulumi.export(f"{name}-schema-discoverer-arn", bus_schema_discoverer.arn)
 
+        return bus.name
+
 
     @classmethod
     def create_rule_target(cls,
@@ -131,7 +128,6 @@ class Events:
             optional_log_group (Optional[bool]): True or False.  If True, this will create a Cloudwatch Log group that can be used to troubleshoot event data.  Ideally used for debug and troubleshooting.
 
         """
-
         resource_name = aws.get_arn(arn=target_arn).resource
         resource_type = aws.get_arn(arn=target_arn).service
 
@@ -152,20 +148,18 @@ class Events:
             # If so, do a json dump to string
             event_pattern = Events._dict_to_json(value=event_pattern, value_name="event_pattern")
 
-        # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
-        rule = aws.cloudwatch.EventRule(
-            f"{name}-rule",
-            name = name,
-            description = description,
-            event_bus_name = event_bus_name,
-            is_enabled = True,
-            event_pattern = event_pattern
-        )
-
-        pulumi.export(f"{name}-rule-arn", rule.arn)
-
-
         def general_event():
+            # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+            rule = aws.cloudwatch.EventRule(
+                f"{name}-rule",
+                name = name,
+                description = description,
+                event_bus_name = event_bus_name,
+                is_enabled = True,
+                event_pattern = event_pattern
+            )
+            pulumi.export(f"{name}-rule-arn", rule.arn)
+
             # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventtarget/
             aws.cloudwatch.EventTarget(
                 f"{name}-rule-target",
@@ -180,10 +174,23 @@ class Events:
                 opts = pulumi.ResourceOptions(parent=rule)
             )
 
+            return rule.arn
+
         def create_lambda_event():
             # Additional Logic for Lambda as a Target
             print("Lambda Target --> Adding Lambda Invoke Permission")
 
+            # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+            rule = aws.cloudwatch.EventRule(
+                f"{name}-rule",
+                name = name,
+                description = description,
+                event_bus_name = event_bus_name,
+                is_enabled = True,
+                event_pattern = event_pattern
+            )
+            pulumi.export(f"{name}-rule-arn", rule.arn)
+
             # https://www.pulumi.com/registry/packages/aws/api-docs/lambda/permission/
             aws.lambda_.Permission(
                 f"{name}-lambda-permission",
@@ -207,6 +214,8 @@ class Events:
                 opts = pulumi.ResourceOptions(parent=rule)
             )
 
+            return rule.arn
+
         def create_events_event():
 
             # This could be an API Destination or trying to send to another Eventbridge
@@ -214,6 +223,17 @@ class Events:
                 # Additional Logic for another Event Bus as a Target
                 print("Event Bus Target --> Creating IAM Role")
 
+                # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+                rule = aws.cloudwatch.EventRule(
+                    f"{name}-rule",
+                    name = name,
+                    description = description,
+                    event_bus_name = event_bus_name,
+                    is_enabled = True,
+                    event_pattern = event_pattern
+                )
+                pulumi.export(f"{name}-rule-arn", rule.arn)
+
                 assume_role_policy = aws.iam.get_policy_document(statements=[
                     aws.iam.GetPolicyDocumentStatementArgs(
                         actions = ["sts:AssumeRole"],
@@ -245,6 +265,7 @@ class Events:
                     )],
                     assume_role_policy_document = assume_role_policy.json
                 )
+                pulumi.export(f"{name}-iam-role-arn", event_bus_role.arn)
 
                 # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventtarget/
                 aws.cloudwatch.EventTarget(
@@ -261,13 +282,24 @@ class Events:
                     opts = pulumi.ResourceOptions(parent=rule)
                 )
 
-                pulumi.export(f"{name}-iam-role-arn", event_bus_role.arn)
+                return rule.arn
 
             else:
                 # Additional Logic for API Destination as a Target
                 print("API Destination Target --> Creating IAM Role")
                 print("API Destination Target --> Adding Invoke API Permission")
 
+                # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+                rule = aws.cloudwatch.EventRule(
+                    f"{name}-rule",
+                    name = name,
+                    description = description,
+                    event_bus_name = event_bus_name,
+                    is_enabled = True,
+                    event_pattern = event_pattern
+                )
+                pulumi.export(f"{name}-rule-arn", rule.arn)
+
                 assume_role_policy = aws.iam.get_policy_document(statements=[
                     aws.iam.GetPolicyDocumentStatementArgs(
                         actions = ["sts:AssumeRole"],
@@ -299,6 +331,7 @@ class Events:
                     )],
                     assume_role_policy_document = assume_role_policy.json
                 )
+                pulumi.export(f"{name}-iam-role-arn", destination_role.arn)
 
                 # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventtarget/
                 aws.cloudwatch.EventTarget(
@@ -315,12 +348,23 @@ class Events:
                     opts = pulumi.ResourceOptions(parent=rule)
                 )
 
-                pulumi.export(f"{name}-iam-role-arn", destination_role.arn)
+                return rule.arn
 
         def create_step_function_event():
             # Additional Logic for Lambda as a Target
             print("Step Function Target --> Invoke State Machine")
 
+            # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+            rule = aws.cloudwatch.EventRule(
+                f"{name}-rule",
+                name = name,
+                description = description,
+                event_bus_name = event_bus_name,
+                is_enabled = True,
+                event_pattern = event_pattern
+            )
+            pulumi.export(f"{name}-rule-arn", rule.arn)
+
             assume_role_policy = aws.iam.get_policy_document(statements=[
                 aws.iam.GetPolicyDocumentStatementArgs(
                     actions = ["sts:AssumeRole"],
@@ -352,6 +396,7 @@ class Events:
                 )],
                 assume_role_policy_document = assume_role_policy.json
             )
+            pulumi.export(f"{name}-iam-role-arn", step_function_role.arn)
 
             # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventtarget/
             aws.cloudwatch.EventTarget(
@@ -368,12 +413,23 @@ class Events:
                 opts = pulumi.ResourceOptions(parent=rule)
             )
 
-            pulumi.export(f"{name}-iam-role-arn", step_function_role.arn)
+            return rule.arn
 
         def create_kinesis_stream_event():
             # Additional Logic for Kinesis Stream as a Target
             print("Kinesis Stream Target --> Put Record on Kinesis Stream")
 
+            # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+            rule = aws.cloudwatch.EventRule(
+                f"{name}-rule",
+                name = name,
+                description = description,
+                event_bus_name = event_bus_name,
+                is_enabled = True,
+                event_pattern = event_pattern
+            )
+            pulumi.export(f"{name}-rule-arn", rule.arn)
+
             assume_role_policy = aws.iam.get_policy_document(statements=[
                 aws.iam.GetPolicyDocumentStatementArgs(
                     actions = ["sts:AssumeRole"],
@@ -405,6 +461,7 @@ class Events:
                 )],
                 assume_role_policy_document = assume_role_policy.json
             )
+            pulumi.export(f"{name}-iam-role-arn", kinesis_role.arn)
 
             # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventtarget/
             aws.cloudwatch.EventTarget(
@@ -421,13 +478,24 @@ class Events:
                 opts = pulumi.ResourceOptions(parent=rule)
             )
 
-            pulumi.export(f"{name}-iam-role-arn", kinesis_role.arn)
+            return rule.arn
 
         def create_queue_event():
             # Additional Logic for Lambda as a Target
             print("SQS Target --> Adding Queue Policy")
 
-            queue_url = aws.sqs.get_queue(name=resource_name).url
+            sqs_queue = aws.sqs.get_queue(name=resource_name)
+
+            # https://www.pulumi.com/registry/packages/aws/api-docs/cloudwatch/eventrule/
+            rule = aws.cloudwatch.EventRule(
+                f"{name}-rule",
+                name = name,
+                description = description,
+                event_bus_name = event_bus_name,
+                is_enabled = True,
+                event_pattern = event_pattern
+            )
+            pulumi.export(f"{name}-rule-arn", rule.arn)
 
             # https://www.pulumi.com/registry/packages/aws/api-docs/iam/getpolicydocument/
             queue_policy = aws.iam.get_policy_document(
@@ -455,7 +523,7 @@ class Events:
             # https://www.pulumi.com/registry/packages/aws/api-docs/sqs/queuepolicy/
             aws.sqs.QueuePolicy(
                 f"{name}-queue-policy",
-                queue_url=queue_url,
+                queue_url=sqs_queue.url,
                 policy=queue_policy.json,
                 opts = pulumi.ResourceOptions(parent=rule)
             )
@@ -474,6 +542,8 @@ class Events:
                 opts = pulumi.ResourceOptions(parent=rule)
             )
 
+            return rule.arn
+
         # Conditional add a Cloudwatch Log group and output events to it
         # This is typically only recommended for troubleshooting and not
         # recommended to be on all the time and defaults to only keeping
@@ -502,7 +572,6 @@ class Events:
                 ),
                 opts = pulumi.ResourceOptions(parent=log_group)
             )
-
             pulumi.export(f"{name}-logs-arn", log_group.arn)
 
         # Dictionary used to figure out what type of event and what needs to be run
@@ -513,5 +582,61 @@ class Events:
             "kinesis": create_kinesis_stream_event,
             "sqs": create_queue_event
         }
-
         event_dict.get(resource_type, general_event)()
+
+    @classmethod
+    def create_events_logs_reource_policy(cls,
+        name: str
+    ):
+        """
+            When CloudWatch Logs is the target of a rule, EventBridge creates log streams, and
+            CloudWatch Logs stores the text from the events as log entries. To allow EventBridge to
+            create the log stream and log the events, CloudWatch Logs must include a resource-based
+            policy that enables EventBridge to write to CloudWatch Logs.
+
+        URLs:
+            https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-use-resource-based.html#eb-cloudwatchlogs-permissions
+
+        Args:
+            name (str): Name that will be used to create the resource policy
+        """
+
+        account_id = aws.get_caller_identity().account_id
+        region = aws.get_region().name
+
+        resource_policy = aws.cloudwatch.LogResourcePolicy.get("resource_policy_lookup",
+            id=name
+        )
+
+        # print(resource_policy.policy_name)
+        # print(resource_policy.policy_document)
+
+        # https://www.pulumi.com/registry/packages/aws/api-docs/iam/getpolicydocument/
+        eventbridge_to_cloudwatch_logs = aws.iam.get_policy_document(
+            statements=[
+                aws.iam.GetPolicyDocumentStatementArgs(
+                    sid=name,
+                    actions = [
+                        "logs:CreateLogStream",
+                        "logs:PutLogEvents"
+                    ],
+                    principals=[
+                        aws.iam.GetPolicyDocumentStatementPrincipalArgs(
+                            type="Service",
+                            identifiers=[
+                                "events.amazonaws.com",
+                                "delivery.logs.amazonaws.com"
+                            ],
+                        )
+                    ],
+                    resources = [f"arn:aws:logs:{region}:{account_id}:log-group:/aws/events/*:*"],
+                )
+            ]
+        )
+
+        cloudwatch_resource_policy = aws.cloudwatch.LogResourcePolicy(
+            f"{name}-cloudwatch-resource-policy",
+            policy_name=name,
+            policy_document=eventbridge_to_cloudwatch_logs.json,
+        )
+        return cloudwatch_resource_policy.policy_name
